@@ -9,11 +9,15 @@ import logging
 import json
 from time import sleep
 
+from tuxput import __version__
+
 
 from tpcli.exceptions import (
     UnknownTokenError,
     NoAuthorizationError,
     FileConflictError,
+    BadMethodError,
+    BucketNotFoundError,
 )
 
 UPLOAD_QUEUE = []
@@ -22,6 +26,8 @@ RESULTS = {}
 RETRIES = 1
 WAITTIME = 10
 
+logging.basicConfig(format="%(levelname)s: %(message)s")
+
 
 def good_response(code):
     if code in [200, 204]:
@@ -72,12 +78,12 @@ def get_presigned_url(url, token, upload_file, bucket=None):
     headers = {"X-tuxput-token": token}
     if bucket is not None:
         headers["X-tuxput-bucket"] = bucket
-    full_url = url + "/" + upload_file
+    full_url = url + upload_file
     while retries > 0:
         retries -= 1
 
         logging.info(
-            "sending req to {0}/{1} -- attempt {2}/{3}".format(
+            "sending req to {0}{1} -- attempt {2}/{3}".format(
                 url, upload_file, retries, RETRIES
             )
         )
@@ -97,6 +103,12 @@ def get_presigned_url(url, token, upload_file, bucket=None):
         elif resp.status_code == 403:
             logging.debug("raising NoAuthorizationError")
             raise NoAuthorizationError
+        elif resp.status_code == 404:
+            logging.debug("raising BucketNotFoundError")
+            raise BucketNotFoundError
+        elif resp.status_code == 405:
+            logging.debug("raising BadMethodError")
+            raise BadMethodError("{0} ({1})".format(resp.reason, resp.url))
         # file already exists
         elif resp.status_code == 409:
             logging.debug("raising FileConflictError")
@@ -129,37 +141,116 @@ def do_s3_upload(presigned, upload_file):
     return resp
 
 
+def print_usage():
+    ctx = click.get_current_context()
+    click.echo()
+    click.echo(ctx.get_help())
+    ctx.exit()
+
+
 @click.command()
-@click.argument("url", nargs=1)
+@click.argument("url", nargs=1, required=False)
 @click.argument("targets", nargs=-1)
-@click.option("--dry-run", "-n", default=False, is_flag=True)
-@click.option("--token", "-t", required=True, type=str)
-@click.option("--bucket", "-b", required=False, type=str)
 @click.option(
-    "--list_buckets", "-l", required=False, default=False, is_flag=True
+    "--dry-run",
+    "-n",
+    default=False,
+    is_flag=True,
+    help="show files that would be upload, but do not actually upload",
 )
 @click.option(
-    "--list_auths", "-A", required=False, default=False, is_flag=True
+    "--token",
+    "-t",
+    required=False,
+    type=str,
+    envvar="TPCLI_TOKEN",
+    help="identification token for the tuxput server",
+)
+@click.option(
+    "--bucket",
+    "-b",
+    required=False,
+    type=str,
+    envvar="TPCLI_BUCKET",
+    help="name of the S3 bucket for the upload",
+)
+@click.option("--version", "-V", required=False, is_flag=True)
+@click.option(
+    "--list_buckets",
+    "-l",
+    required=False,
+    default=False,
+    is_flag=True,
+    help="list the S3 buckets available through tuxput",
+)
+@click.option(
+    "--list_auths",
+    "-A",
+    required=False,
+    default=False,
+    is_flag=True,
+    help="list the authorized buckets and paths for this token",
+)
+@click.option(
+    "--sign-only",
+    "-s",
+    default=False,
+    is_flag=True,
+    help="generate and print signed urls for each TARGET, but do not upload",
 )
-@click.option("--sign-only", "-s", default=False, is_flag=True)
 def tuxput_cli(
-    url, targets, dry_run, token, bucket, list_buckets, list_auths, sign_only
+    url,
+    targets,
+    dry_run,
+    token,
+    bucket,
+    version,
+    list_buckets,
+    list_auths,
+    sign_only,
 ):
 
+    if version:
+        print("{0} version {1}".format(__name__, __version__))
+        return
+
+    # Everything after this point requires the url to be specified
+    if url is None:
+        logging.error("url is required")
+        print_usage()
+
+    # Let's make sure url ends with a / to keep things consistent
+    if url[-1] != "/":
+        url = url + "/"
+
     if list_buckets:
         print(list_server_buckets(url, token))
         return
 
+    # Everything after this point requires a token
+    if token is None:
+        logging.error("token is required")
+        print_usage()
+
     if list_auths:
         print(list_server_authorizations(url, token))
         return
 
+    if dry_run:
+        logging.getLogger().setLevel(logging.INFO)
+
+    # technically not an error, but should let the user know
+    # that they didn't give us anything to upload
+    if not targets:
+        logging.error("no upload targets were specified")
+        print_usage()
+
     # collect the files we'll be uploading
     for t in targets:
         UPLOAD_QUEUE.extend(scan_for_files(t))
 
     for f in UPLOAD_QUEUE:
-        logging.info("-- Processing {0}".format(f))
+        logging.info("processing {0}".format(f))
 
         if dry_run:
             logging.debug("dry-run enabled... skipping")
@@ -173,7 +264,7 @@ def tuxput_cli(
                 # the fail and move on.
                 RESULTS[
                     f
-                ] = "Failed to get presigned URL:  {0}{1}... skipping".format(
+                ] = "Failed to get presigned URL: {0} {1}... skipping".format(
                     f, presigned.status_code
                 )
                 continue
@@ -193,6 +284,14 @@ def tuxput_cli(
         except FileConflictError:
             RESULTS[f] = "{0} file already exists".format(f)
             continue
+        except BucketNotFoundError:
+            # if bucket isn't found, no since in continuing
+            logging.error("bucket not found: {0}".format(bucket))
+            sys.exit(1)
+        except BadMethodError as e:
+            # server doesn't recognize the endpoint, so bail
+            logging.error(e.msg)
+            sys.exit(1)
         except NoAuthorizationError:
             # Not necessarily fatal.  Log the fail and move on.
             RESULTS[f] = "no authorization to get presigned url"
@@ -216,7 +315,7 @@ def tuxput_cli(
 
     print("Results\n--------------")
     for x in RESULTS:
-        print("{0} -{1}".format(x, RESULTS[x]))
+        print("{0} - {1}".format(x, RESULTS[x]))
 
 
 # TODO possibly in version 2.0?
@@ -226,4 +325,4 @@ def tuxput_cli(
 
 
 if __name__ == "__main__":
-    tuxput_cli(auto_envvar_prefix="TUXPUT_CLI")
+    tuxput_cli(auto_envvar_prefix="TPCLI")
