@@ -9,7 +9,6 @@ class TuxConfig:
         self.config_strings = {}
         self.config_strings["S3_REGION"] = None
         self.config_strings["CONF_BUCKET"] = None
-        self.config_strings["DEFAULT_BUCKET"] = None
         self.config_strings["CONF_FILE"] = "tuxput.json"
 
         self.config_bools = {}
@@ -34,11 +33,6 @@ class TuxConfig:
             self.config_strings["S3_REGION"] is not None
         ), "Required env var S3_REGION not set"
 
-        if "DEFAULT_BUCKET" not in self.config_strings:
-            self.config_strings["DEFAULT_BUCKET"] = self.config_strings[
-                "CONF_BUCKET"
-            ]
-
     @property
     def S3_REGION(self):
         return self.config_strings["S3_REGION"]
@@ -58,7 +52,3 @@ class TuxConfig:
     @property
     def CONF_FILE(self):
         return self.config_strings["CONF_FILE"]
-
-    @property
-    def DEFAULT_BUCKET(self):
-        return self.config_strings["DEFAULT_BUCKET"]
